#!/bin/bash

#load and parse sitedata.yaml for vars
source yaml-parser.sh
eval "$(parse_yaml "sitedata.yaml")"

GREEN='\033[0;32m'
RED='\033[0;31m'
GREY='\e[93m'
NORMAL='\e[0m'

message() {
    echo -e "$GREEN${1}...$GREEN"
}

ignore_message() {
    echo -e "$RED ${@} $RED"
}

task_message() {
    echo -e "$GREY[+] $GREEN ${task} $NORMAL"
}

check_lines_in_file() {
    COUNT=$(grep "$CONFIG" $FILE 2>/dev/null | wc -l)
    echo -e "\t$COUNT: $CONFIG"
    if [ 0 -eq $COUNT ]; then
        echo "$CONFIG" >>$FILE
    else
        echo "IGNORING: $CONFIG exists in $FILE"
    fi
}

check_file_for_rule() {
    if grep "^$RULE$" $FILE 2>/dev/null; then
        ignore_message "$RULE exists in $FILE"
    else
        echo "$RULE" >>$FILE
    fi
}


#===========================================================
################# FILESYSTEM CONFIGURATION #################
#===========================================================

##1.1.1.x Stop auto mounting unused filesystems (Scored)
remove_unused_filesystem() {
    task="Removing unused filesystem"
    task_message
    # task_message "Removing unused filesystem"
    for CONFIG in \
        'install cramfs /bin/true' \
        'install freevxfs /bin/true' \
        'install jffs2 /bin/true' \
        'install hfs /bin/true' \
        'install hfsplus /bin/true' \
        'install udf /bin/true' \
        'install dccp /bin/true' \
        'install sctp /bin/true' \
        'install vfat /bin/true' \
        'install rds /bin/true' \
        'install tipc /bin/true' \
        'install squashfs /bin/true' \
        'install mtd /bin/true'; do
        FILE=/etc/modprobe.d/cis.conf
        check_before_adding=$(check_lines_in_file)
    done

    for i in \
        'cramfs' \
        'freevxfs' \
        'jffs2' \
        'hfs' \
        'hfsplus' \
        'squashfs' \
        'udf'; do
        rmmod $i 2>/dev/null
    done
}

#1.1.20 Ensure sticky bit is set on all world-writable directories (Scored)
configure_sticky_bit() {
    task="Configuring Sticky bit on all world-writable directories"
    task_message
    #Prevents the ability to rename files in world writable directories such as /tmp (owned by the user)
    df --local -P | awk {'if (NR!=1) print $6'} | xargs -I '{}' find '{}' -xdev -type d -perm -0002 2>/dev/null | xargs chmod a+t
}

#Ensure aide tool is installed
install_aide_tool() {
    #AIDE takes a snapshot of filesystem state and modification times. It can detect changes
    task="Initialising aide"
    task_message
    aideinit
    cp /var/lib/aide/aide.db.new /var/lib/aide/aide.db
    update-aide.conf
    cp /var/lib/aide/aide.conf.autogenerated /etc/aide/aide.conf
    configure_mta_local_only_node
}

## This section assumes a seperate partition have been specified for /var
## No exec on /dev/shm, stops binary files from being executed from /tmp
## The nodev option ensures filesystem cannot contain special devices.
## The nosuid option ensures filesystem cannot contain setuid files, it stops the introduction of
## privileged programs that can be executed by non-root users

#1.1.3 Ensure nodev option set on /tmp partition (Scored)
#1.1.4 Ensure nosuid option set on /tmp partition (Scored)
#1.1.14 Ensure nodev option set on /dev/shm partition (Scored)
#1.1.15 Ensure nosuid option set on /dev/shm partition (Scored)
#1.1.16 Ensure noexec option set on /dev/shm partition (Scored)

filesystem_mount_options() {
    task="Configuring filesystem options"
    task_message

    sed -i 's/\/var\/tmp ext4.*/\/var\/tmp ext4 defaults,nodev,nosuid,noexec  0 0/' /etc/fstab
    sed -i 's/\/home ext4.*/\/home ext4 defaults,nodev  0  0/' /etc/fstab

    for CONFIG in \
        'tmpfs    /dev/shm    tmpfs    defaults,nodev,nosuid,noexec   0 0' \
        'tmpfs    /tmp    tmpfs    defaults,nodev,nosuid,noexec   0 0'; do
        FILE=/etc/fstab
        check_before_adding=$(check_lines_in_file)
    done
    mount -av
}

## Periodic check of filesystem integrity
## AIDE Filesystem Check, crontab alerts when unauthorised changes to conf files are detected
#1.3.2 Ensure filesystem integrity is regularly checked scored

file_system_integrity_check() {
    task="Creating ADIE Crontab job for periodic filesystem integrity checks"
    task_message
    if egrep -q "^0\s+5\s+\*\s+\*\s+\*\s+\/usr\/bin\/aide\.wrapper\s+\-\-config\s+\/etc\/aide\/aide\.conf\s+\-\-check$" /var/spool/cron/crontabs/root 2>/dev/null; then
        ignore_message "IGNORING: This crontab job exists"
    else
        (
            crontab -l
            echo "0 5 * * * /usr/bin/aide.wrapper --config /etc/aide/aide.conf --check"
        ) | crontab -
    fi
}

#=======================================================
################# SECURE BOOT SETTINGS #################
#=======================================================

## Grub superusers and password, ensures that password is required to set command line boot
## parameters during system reboot.
## Requiring a boot password upon execution of the boot loader will prevent an unauthorized user from
## entering boot parameters or changing the boot partition.
#1.4.2 Ensure bootloader password is set (Scored)

set_bootloader_passwd() {
    task='Configuring bootloader password'
    task_message
    for CONFIG in \
        "set superusers=\"${btuser}\"" \
        "password_pbkdf2 ${btuser} ${grubpassword}"; do
        FILE=/etc/grub.d/40_custom
        check_and_apply=$(check_lines_in_file)
    done

    #TODO: run test to see what happens in single user mode
    #1.4.3 Ensure authentication required for single user mode (Scored)
    #task='Configuring authentication for single user mode'
    #task_message
    #sed --in-place "/^root:\!:18219:0:99999:7:::$/d" /etc/shadow
    #CONFIG='root:$6$Xs5Ny.Il$lZ.M08SxKPYI9h7q/Ir27dO5FQSCgDT8nA7T3WO2GWerJFqjsvX43heJwW5Tq9mZgfqH2.IfYazB04NZMavHS0:18418:0:99999:7:::'
    #FILE=/etc/shadow
    #check_and_apply=$(check_lines_in_file)

    ## Add --unrestricted to grub menu entry to allow the system to boot without entering a password.
    ## Password will still be required to edit menu items
    task="Adding \"--unrestricted\" to grub menu entry"
    task_message
    if grep -q 'echo "menuentry '\''$(echo "$os" | grub_quote)'\'' ${CLASS} --unrestricted' /etc/grub.d/10_linux; then
        ignore_message 'Line exists in /etc/grub.d/10_linux'
    else
        sed -i.bak 's/\(\"$os\" | grub_quote)'\'' ${CLASS}\)/\0 --unrestricted/' /etc/grub.d/10_linux
    fi

    ##When grub is updated, the user/password information is automatically added to the GRUB 2 menu
    ##configuration file, grub.cfg. Reboot system to verify functionality.

    update-grub
}

#===============================================================
################# ADDITIONAL PROCESS HARDENING #################
#===============================================================

## Setting the fs.suid_dumpable to 0 will prevent setuid programs from dumping core.
## Setting kernel.randomize_va_space=2 makes it difficult to write memory page exploits as the
## memory placement will shift consistently.

#1.5.x Ensure core dumps are restricted (Scored)
## Setting the hard limit on core dumps to 0 prevents the users from overriding the soft variable.
## Setting the fs.suid_dumpable variable to 0 will prevent setuid programs from dumping core.

core_dumps_and_additional_hardening() {
    task='Ensure core dumps are restricted'
    task_message

    if grep -q "* hard core 0" /etc/security/limits.conf; then
        ignore_message 'Line exists in file /etc/security/limits.conf'
    else
        echo "* hard core 0" >>/etc/security/limits.conf
    fi

    #3.1.x Disable Packet Redirect Sending
    #3.3 Disable All IPv6
    #3.5.x Incomming Network Protocols

    task='Configuring network hardening'
    task_message
    echo -n > /etc/sysctl.conf
    for CONFIG in \
        'fs.suid_dumpable = 0' \
        'kernel.sysrq = 0' \
        'kernel.randomize_va_space = 2' \
        'kernel.core_uses_pid = 1' \
        'kernel.dmesg_restrict = 1' \
        'kernel.kptr_restrict = 2' \
        'net.ipv4.ip_forward = 0' \
        'net.ipv4.conf.all.forwarding = 0' \
        'net.ipv4.conf.all.send_redirects = 0' \
        'net.ipv4.conf.default.send_redirects = 0' \
        'net.ipv4.conf.all.accept_redirects = 0' \
        'net.ipv4.conf.default.accept_redirects = 0' \
        'net.ipv4.conf.all.accept_source_route = 0' \
        'net.ipv4.conf.default.accept_source_route = 0' \
        'net.ipv4.conf.all.secure_redirects = 0' \
        'net.ipv4.conf.default.secure_redirects = 0' \
        'net.ipv4.conf.all.log_martians = 1' \
        'net.ipv4.conf.default.log_martians = 1' \
        'net.ipv4.icmp_echo_ignore_broadcasts = 1' \
        'net.ipv4.icmp_ignore_bogus_error_responses = 1' \
        'net.ipv4.conf.all.rp_filter = 1' \
        'net.ipv4.conf.default.rp_filter = 1' \
        'net.ipv4.tcp_syncookies = 1' \
        'net.ipv6.conf.all.forwarding = 0' \
        'net.ipv6.conf.all.accept_source_route = 0' \
        'net.ipv6.conf.default.accept_source_route = 0' \
        'net.ipv6.conf.all.accept_ra = 0' \
        'net.ipv6.conf.default.accept_ra = 0' \
        'net.ipv6.conf.all.accept_redirects = 0' \
        'net.ipv6.conf.default.accept_redirects = 0'; do
        FILE=/etc/sysctl.conf
        check_before_applying=$(check_lines_in_file)
    done
    sed -i 's~echo 2 > /proc/sys/fs/suid_dumpable~#echo 2 > /proc/sys/fs/suid_dumpable~' /etc/init.d/apport
    sysctl -p /etc/sysctl.conf 2>/dev/null
}

#===================================================
################# WARNING BARNNERS #################
#===================================================

# 1.7 Warning Banners
## The warning banner is configured to have no instances \m - machine architecture, \r - operating
## system release, \s - operating system name and \v - operating system version
config_warning_banner() {
    task="Creating BT Banner"
    task_message
    tee /etc/issue /etc/issue.net >/dev/null <<EOF
##########################################################################

                British Telecommunications plc.

WARNING: You have accessed a computer managed by BT.
You are required to have authorisation from BT before you proceed
and you are strictly limited to the use set out within that authorisation.
Unauthorised access to or misuse of this system or the data contained on
this system is strictly prohibited and may constitute an offence.

If you disclose any data or other information obtained through this system
without authority, BT will take appropriate action against you. This may
include legal proceedings, prosecution and disciplinary action up to and
including dismissal.

##########################################################################


EOF
}

#===========================================
################# SERVICES #################
#===========================================

# 2.2.15 Ensure mail transfer agent is configured for local-only mode (Scored)
#This line does nothing, the intention is to stop MTA from listening on any non loopback interface.
configure_mta_local_only_node() {
    task='Configuring MTA local only mode'
    task_message
    echo "inet_interfaces = loopback-only
inet_protocols = ipv4
myhostname = server.example.com
smtpd_banner = \$myhostname ESMTP" >>/etc/postfix/main.cf
    systemctl restart postfix
}

## 2.2.16 Ensure rsync service is not enabled (Scored)
#rsync is used to synchronise files between systems but it uses unencrypted protocols for communication.

disable_rsync() {
    task='disabling rsync'
    task_message
    systemctl disable rsync
}

# 2.3.4 Ensure telnet client is not installed (Scored)
purge_telnet() {
    task='Purging telnet protocol'
    task_message
    apt purge -y telnet
}

# 3.3.3 Disable IPv6
## If IPv6 is not to be used, it is recommended that it be disabled to reduce the attack surface of the system.
# 4.1.3 Ensure auditing for processes that start prior to auditd is enabled (Scored)
## The above ensures processes that are capable of being audited can be audited even if they start up prior to auditd startup.
update_grub_config() {
    task='Updating grub config'
    task_message
    sed -i 's/^GRUB_CMDLINE_LINUX=.*/GRUB_CMDLINE_LINUX="console=tty1 console=ttyS0,115200n8 ipv6.disable=1 audit=1 audit_backlog_limit=8192"/' /etc/default/grub
    update-grub || ignore_message "GRUB UPDATE FAILED"
}

remove_insecure_clients() {
    task="Purging insecure clients: rsh, rcp and rlogin"
    task_message
    apt purge --auto-remove rsh-server rsh-client
    for i in rsh rcp rlogin; do
        rm -r /usr/bin/${i} /etc/alternatives/${i}* 2>/dev/null
    done
}

#====================================================================
################# CONNECTIVITY/ALLOWED HOSTS CONFIG #################
#====================================================================

# 3.4.2|3 Configure hosts.allow|deny
## Defines IPs that can connect to the device.

config_hosts_file() {
   task='Configuring /etc/hosts file'
   task_message
   file='/etc/hosts'
   echo "" >> $file
   for line in \
   "# VPC HOSTS" \
   "$snapip $snapdns" \
   "$landscapeip $landscapedns" \
   "$checkmkip $checkmkdns"; do
   echo $line >> $file
   done
}

configure_allowed_hosts() {
    task='Configuring allowed hosts'
    task_message
    allow_file='/etc/hosts.allow'
    echo -n > $allow_file
    for line in \
    "sshd: $snapdns" \
    'sshd: ALL: deny' \
    "ALL: $landscapedns"; do
    echo $line >> $allow_file
    done
    echo 'ALL: ALL' > /etc/hosts.deny
}

#==================================================
################# FIREWALL POLICY #################
#==================================================

# 3.6.2 Deny Firewall Policy
# 3.6.3 Configure Loopback Trafic
# 3.6.5 Firewall for open ports
## Reply to Ping configured
##This section assumes that the /etc/iptables directory does not exists by defualt

configure_firewall_policy() {
    task='Applying Firewall Policy'
    task_message

    # Flush Iptables rules
    iptables -F

    # Default deny Firewall policy
    iptables -P INPUT DROP
    iptables -P OUTPUT DROP
    iptables -P FORWARD DROP

    # Ensure loopback traffic is configured
    iptables -A INPUT -i lo -j ACCEPT
    iptables -A OUTPUT -o lo -j ACCEPT
    iptables -A INPUT -s 127.0.0.0/8 -j DROP

    # Ensure outbound and established connections are configured
    iptables -A OUTPUT -p tcp -m state --state NEW,ESTABLISHED -j ACCEPT
    iptables -A OUTPUT -p udp -m state --state NEW,ESTABLISHED -j ACCEPT
    iptables -A OUTPUT -p icmp -m state --state NEW,ESTABLISHED -j ACCEPT
    iptables -A INPUT -i $devicemgtinterface -p icmp -m state --state NEW,ESTABLISHED -j ACCEPT
    iptables -A INPUT -i $devicemgtinterface -p tcp --dport 6556 -s $checkmkip -m state --state NEW,ESTABLISHED -j ACCEPT
    iptables -A INPUT -i $devicemgtinterface -p tcp --dport 6556 -j DROP
    iptables -A INPUT -i $devicemgtinterface -p tcp -s $awssubnet -m state --state NEW,ESTABLISHED -j ACCEPT
    iptables -A INPUT -i $devicemgtinterface -p tcp -s $devicemgtsubnet -m state --state NEW,ESTABLISHED -j ACCEPT
   # iptables -A INPUT -i $DATA_IFACE -p tcp -s $DATA_SUBNET -m state --state NEW,ESTABLISHED -j ACCEPT
    


    DIRECTORY=/etc/iptables
    if [ ! -d "$DIRECTORY" ]; then
        mkdir "$DIRECTORY"
        sudo touch ${DIRECTORY}/rules.v4
        sudo touch ${DIRECTORY}/apply.sh
    fi
    cat <<EOF >/etc/iptables/rules.v4
*filter
:INPUT DROP [0:0]
:FORWARD DROP [0:0]
:OUTPUT DROP [0:0]
-A INPUT -i lo -j ACCEPT
-A OUTPUT -o lo -j ACCEPT
-A INPUT -s 127.0.0.0/8 -j DROP
-A INPUT -i $devicemgtinterface -p icmp -m state --state NEW,ESTABLISHED -j ACCEPT
-A INPUT -i $devicemgtinterface -p tcp --dport 6556 -s $checkmkip -m state --state NEW,ESTABLISHED -j ACCEPT
-A INPUT -i $devicemgtinterface -p tcp --dport 6556 -j DROP
-A INPUT -i $devicemgtinterface -p tcp -s $awssubnet -m state --state NEW,ESTABLISHED -j ACCEPT
-A INPUT -i $devicemgtinterface -p tcp -s $devicemgtsubnet -m state --state NEW,ESTABLISHED -j ACCEPT
-A OUTPUT -p tcp -m state --state NEW,ESTABLISHED -j ACCEPT
-A OUTPUT -p udp -m state --state NEW,ESTABLISHED -j ACCEPT
-A OUTPUT -p icmp -m state --state NEW,ESTABLISHED -j ACCEPT
COMMIT
# Completed on Wed Aug 21 16:41:40 2019
EOF

    ## Apply Config at boot
    ## Required if iptables-persistent is not installed (install during system setup?)

    for CONFIG in \
        '#!/bin/sh' \
        '/sbin/iptables-restore < /etc/iptables/rules.v4'; do
        FILE=/etc/iptables/apply.sh
        check_before_applying=$(check_lines_in_file $CONFIG $FILE)
    done

    chmod +x /etc/iptables/apply.sh
    if egrep -q "^@reboot /etc/iptables/apply.sh  >/var/log/iptablesonboot.log 2>&1" /var/spool/cron/crontabs/root 2>/dev/null; then
        ignore_message "IGNORING: This Cron job exists"
    else
        (
            crontab -l
            echo "@reboot /etc/iptables/apply.sh >/var/log/iptablesonboot.log 2>&1"
        ) | crontab -
    fi
}

#=======================================================
################# Logging and Auditing #################
#=======================================================

# 4.1.1.1 Ensure auditd is installed (Scored)
# 4.1.1.2 Ensure auditd service is enabled (Scored)
# 4.1.1.3 Ensure audit log storage size is configured (Scored)
# 4.1.2.2 Ensure audit logs are not automatically deleted (Scored)
# 4.1.2.3 Ensure system is disabled when audit logs are full (Scored)
# 4.1.3 Ensure auditing for processes that start prior to auditd is enabled (Scored)
## Configure the maximum size of the audit log file. Once the log reaches the maximum size,
## it will be rotated and a new log file will be started.

# Ensure system is disabled when audit logs are full (Scored)

# 4.2.4 Ensure permissions on all logfiles are configured
# 4.1.4 Ensure events that modify date and time information are collected
# 4.1.5 Ensure events that modify user/group information are collected
# 4.1.6 Ensure events that modify the system's network environment are collected
# 4.1.7 Ensure events that modify the system's Mandatory Access Controls are collected
# 4.1.8 Ensure login and logout events are collected
# 4.1.9 Ensure session initiation information is collected
# 4.1.10 Ensure discretionary access control permission modification events are collected
# 4.1.11 Ensure unsuccessful unauthorized file access attempts are collected
# 4.1.13 Ensure successful file system mounts are collected
# 4.1.14 Ensure file deletion events by users are collected
# 4.1.15 Ensure changes to system administration scope (sudoers) is collected
# 4.1.16 Ensure system administrator actions (sudolog) are collected
# 4.1.17 Ensure kernel module loading and unloading is collected
# 4.1.18 Ensure the audit configuration is immutable

## This goes first as the setting `-e 2` need to be last
## Most of this section monitors changes to file permissions, attributes, ownership and group.

configure_auditing() {
    task='Configuring auditd log storage size'
    task_message
    #Once the max size(mb) is reached, the log is kept and a new log is started, see man auditd for info.
    cp auditd-cis.conf /etc/audit/auditd.conf
    systemctl enable auditd
    service auditd restart

}

apply_audit_rules() {
    task='Applying Audit rules'
    task_message
    cp audit-cis.rules /etc/audit/audit.rules

    find / -xdev \( -perm -4000 -o -perm -2000 \) -type f | awk '{print \
    "-a always,exit -F path=" $1 " -F perm=x -F auid>=1000 -F auid!=4294967295 \
    -k privileged" } ' >>/etc/audit/audit.rules

    echo " " >>/etc/audit/audit.rules
    echo "#End of Audit Rules" >>/etc/audit/audit.rules
    echo "-e 2" >>/etc/audit/audit.rules

    cp /etc/audit/audit.rules /etc/audit/rules.d/audit.rules
}

configure_logfiles_permission() {
    task='Configuring logfiles permissions'
    task_message
    chmod -R g-wx,o-rwx /var/log/*
}

## Monitor privileged programs (those that have the setuid and/or setgid bit set on execution) to determine if unprivileged users are running these commands
## Locate all the privileged programs and then add an audit line for each one of them

# 4.2.1.2 Configure rsyslog (Not Scored)
configure_rsyslog() {
    task='Configuring Logging'
    task_message
    for CONFIG in \
        '# Rules for rsyslog' \
        '# For more information see rsyslog.conf(5) and /etc/rsyslog.conf' \
        'auth,authpriv.*                 /var/log/auth.log' \
        '*.*;auth,authpriv.none          -/var/log/syslog' \
        '*.emerg                         :omusrmsg:*' \
        'kern.*                          -/var/log/kern.log' \
        'mail.*                          -/var/log/mail' \
        'mail.info                       -/var/log/mail.info' \
        'mail.warning                    -/var/log/mail.warn' \
        'mail.err                        /var/log/mail.err' \
        'news.crit                       -/var/log/news/news.crit' \
        'news.err                        -/var/log/news/news.err' \
        'news.notice                     -/var/log/news/news.notice' \
        '*.crit                          /var/log/warn' \
        '*.*;mail.none;news.none         -/var/log/messages' \
        'local0,local1.*                 -/var/log/localmessages' \
        'local2,local3.*                 -/var/log/localmessages' \
        'local4,local5.*                 -/var/log/localmessages' \
        'local6,local7.*                 -/var/log/localmessages'; do
        FILE=/etc/rsyslog.d/99-rules.conf
        check_lines_before_applying=$(check_lines_in_file)
    done
}

#============================================================================
################# ACCESS, AUTHENTICATION AND AUTHORISATION  #################
#============================================================================

# 5.1.2 Ensure permissions on /etc/crontab are configured
# 5.1.8 Ensure at/cron is restricted to authorised users (Scored)
## System maintenance jobs that may include security monitoring that have to run, -- cron is used
## to execute them.
## The /etc/crontab file is used by cron to control its own jobs.
## The commands in this item make sure that root is the user and group owner of the file.

configure_cron_permissions() {
    #Configure /etc/cron.allow and /etc/at.allow to allow specific users to use these services.
    task='Configuring Cron permissions'
    task_message
    chmod og-rwx /etc/cron{tab,.hourly,.daily,.weekly,.monthly,.d}
    rm /etc/{cron,at}.deny 2>/dev/null
    echo "$btuser" | tee /etc/cron.allow /etc/at.allow >/dev/null
    chmod og-rwx /etc/{cron,at}.allow
    chown root:root /etc/{cron,at}.allow

}

ssh_config() {
    ## The SSH listen address is not limited to just the IPv4 management interface on this host. /etc/ssh/sshd_config contains the following:
    ## Iptables rules should limit access to host via SSH but removing both lines.
    task='Configuring SSH'
    task_message
    cp sshd_config /etc/ssh/sshd_config
    sed -i -e "s/MGMT_IP/$devicemgtip/g" -e "s/SSH_USERS/$deviceuser $btuser/g"  /etc/ssh/sshd_config
    systemctl restart sshd.service
    chown root:root /etc/ssh/sshd_config
    chmod og-rwx /etc/ssh/sshd_config
}

pam_settings() {
    task='Configuring password quality requirement'
    task_message
    cp common-password /etc/pam.d/common-password

    ## pwquality uses the config below to check password strength. Discusssion for the wider team.
    for CONFIG in \
        'minlen = 14' \
        'dcredit = -1' \
        'ucredit = -1' \
        'ocredit = -1' \
        'lcredit = -1'; do
        FILE=/etc/security/pwquality.conf
        check_before_applying=$(check_lines_in_file)
    done

    ## Locking out user IDs after n unsuccessful consecutive login attempts mitigates brute force password
    ## attacks against your systems.
    ## Users can be unlocked by issuing pam_tally reset command.

    cp common-auth /etc/pam.d/common-auth

}

get_system_users() {
    USERS=$( egrep ^[^:]+:[^\!*] /etc/shadow | cut -d: -f1 )
}

configure_user_passwd_expiry() {
    task='Configuring user password expiry date'
    task_message
    # 5.4.1.x Shadow Password Suite Parameters
    ## Force passwords to expire once they gave reached the defined age
    ## TODO: We need to make a decision on this as it means regular password change.

    sed -i -e 's/\(PASS_MAX_DAYS\t\)[0-9]\+/\1364/g;' \
        -e 's/\(PASS_MIN_DAYS\t\)[0-9]\+/\17/g' /etc/login.defs
        sed -i 's/^UMASK\(\t*\)022/UMASK\1027/' /etc/login.defs

    #This will only impact user accounts created prior to running this script.
    get_system_users
    for username in $USERS; do 
       chage --maxdays 364 $username
       chage --mindays 7 $username
    done
}

inactive_user_passwd_lock() {
    task='Configuring inactive user password lock'
    task_message
    # 5.4.1.4 Ensure inactive password lock is 30 days or less
    ##This is not currently called as it could us out. Do we need it?
    useradd -D -f 30
    get_system_users
    echo $USERS
    for username in $USERS; do
       chage --inactive 30 $username
    done

    #5.4.3 Ensure default group for the root account is GID 0 (Scored)
    usermod -g 0 root
}


set_nologin_on_system() {
    #5.4.2 Ensure system accounts are non-login (Scored)
    #prevent system accounts from providing interactive shells

    task='Ensure system accounts are non-login'
    task_message
    for user in `awk -F: '($3 < 1000) {print $1 }' /etc/passwd`; do
      if [ $user != "root" ]; then
         usermod -L $user
      if [ $user != "sync" ] && [ $user != "shutdown" ] && [ $user != "halt" ]; then
         usermod -s /usr/sbin/nologin $user
      fi
      fi
    done
}

defualt_file_permission_and_shell_timout() {
    # 5.4.4 Default user umask = 027
    # 5.4.5 shell timeout < 900s
    ## Determines the defualt permissions of files created by the user
    ## A defualt permission of 027 grants read permission to users from the same group.
    task='Configuring default user file permissions'
    task_message
    for i in /etc/bash.bashrc /etc/profile $(find /etc/profile.d/ -name "*.sh"); do
        if grep -q '^umask 027$' $i; then
            ignore_message "IGNORING: umask 027 exists $i"
        else
            echo "umask 027" >>$i
        fi
        if grep -q '^TMOUT=900$' $i; then
            ignore_message "IGNORING: TMOUT=900 exists $i"
        else
            echo "TMOUT=900" >>$i
        fi
    done
}

restrict_su_access() {
    # 5.6 Ensure access to the su command is restricted (Scored)
    # Should only allow users in the sudo group
    task='Restricting su access'
    FILE=/etc/pam.d/su
    if egrep "^auth\s+required\s+pam_wheel\.so$" $FILE; then
        ignore_message "IGNORING: rule exists in file: $FILE"
    else
        echo 'auth       required   pam_wheel.so' >>$FILE
    fi
}

# 6.1.3 Ensure permissions on /etc/shadow are configured
# 6.1.5 Ensure permissions on /etc/gshadow are configured
## The /etc/gshadow file is used to store the information about groups that is critical to the
## security of those accounts, such as the hashed password and other security information.
configure_system_file_permissions() {
    task='Configuring system file permissions'
    task_message
    chown root:root /etc/passwd
    chmod 644 /etc/passwd
    chown root:shadow /etc/shadow
    chmod o-rwx,g-wx /etc/shadow
    chown root:root /etc/group
    chmod 644 /etc/group
    chown root:shadow /etc/gshadow
    chmod o-rwx,g-rw /etc/gshadow
    chown root:root /etc/passwd-
    chmod 600 /etc/passwd-
    chown root:root /etc/shadow-
    chmod 600 /etc/shadow-
    chown root:root /etc/group-
    chmod 600 /etc/group-
    chown root:shadow /etc/gshadow-
    chmod o-rwx,g-rw /etc/gshadow-

}

configure_log_file_permissions() {
    task='Configuring log file permissions'
    task_message
    chmod o-rwx $(find /var/log/{lastlog,wtmp,btmp,btmp.1,cloud-init.log})
    chmod o-rwx $(find /var/log/apt/{eipp.log.xz,history.log})
    find /var/log/landscape -type f -exec chmod o-rwx {} \;
}

# 6.2.6 Ensure root PATH integrity (Scored)
## The root user can execute any command on the system and could be fooled into executing programs
## unintentionally if the PATH is not set correctly.
configure_root_path_integrity() {
    task='Configuring root path integrity'
    task_message
    RULE='Defaults secure_path="/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/snap/bin:/sbin:/bin"'
    FILE=/etc/sudoers.d/path
    check_and_apply=$(check_file_for_rule)
    chmod 440 /etc/sudoers.d/path
}

configure_user_home_permission() {
    # 6.2.8 Ensure users' home directories permissions are 750 or more restrictive (Scored)
    ## Group or world-writable user home directories may enable malicious users to steal or
    ## modify other users' data or to gain another user's system privileges.
    chmod 750 /home/*
}


system_reboot() {
    task='System reboot'
    task_message
    if [ -f /var/run/reboot-required ]; then
      echo 'System reboot required: rebooting'
      reboot
    else
      echo 'System reboot not required'
    fi
}

apply_config() {
    remove_unused_filesystem
    install_aide_tool
    filesystem_mount_options
    file_system_integrity_check
    set_bootloader_passwd
    pam_settings
    configure_user_passwd_expiry
    inactive_user_passwd_lock
    set_nologin_on_system
    core_dumps_and_additional_hardening
    config_warning_banner
    disable_rsync
    purge_telnet
    update_grub_config
    remove_insecure_clients
    configure_allowed_hosts
    configure_firewall_policy
    configure_auditing
    apply_audit_rules
    configure_logfiles_permission
    configure_rsyslog
    configure_cron_permissions
    ssh_config
    defualt_file_permission_and_shell_timout
    restrict_su_access
    configure_system_file_permissions
    configure_log_file_permissions
    configure_root_path_integrity
    configure_user_home_permission
}

"$@"
